# coding: utf-8
from django.conf import settings
from pika.exceptions import AMQPConnectionError
from pika.exceptions import ChannelWrongStateError
import pika


class AMQPClient:
    """
    Клиент для взаимодействия по протоколу AMQP.
    """

    def __init__(self, host, queue_name, user='guest', password='guest'):
        """
        Инициализация AMQPClient
        :param host: Хост сервера amqp
        :type host: str
        :param queue_name: Имя очереди
        :type queue_name: str
        :param user: username
        :type user: str
        :param password: password
        :type password: str
        """
        self.host = host
        self.queue_name = queue_name
        self.user = user
        self.password = password
        self._connect()
        self._set_channel()
        self._channel_queue_declare()

    def _reconnect(self):
        """Пробует повторно установить канал связи."""
        self._connect()
        self._set_channel()
        self._channel_queue_declare()

    def _connect(self):
        """Устанавливает соединение с сервером AMQP."""
        self.connection = pika.BlockingConnection(
            pika.ConnectionParameters(
                host=self.host,
                credentials=pika.credentials.PlainCredentials(
                    self.user, self.password
                )
            )
        )

    def close(self):
        """Закрывает соединение с сервером AMQP."""
        self.connection.close()

    def _set_channel(self):
        """Устанавливает канал соединения с сервером AMQP."""
        self.channel = self.connection.channel()

    def _channel_queue_declare(self):
        """
        Устанавливает очередь в канале связи.
        """
        self.channel.queue_declare(
            queue=self.queue_name, durable=True, exclusive=False
        )

    def send_message(self, message):
        """
        Отправка сообщения в очередь, подключенную к серверу AMQP
        :param message: Сообщение
        :type message: Любой тип поддерживающий сериализацию в JSON
        """
        while True:
            try:
                self.channel.basic_publish(
                    exchange='',
                    routing_key=self.queue_name,
                    body=message,
                    properties=pika.BasicProperties(
                        delivery_mode=2,
                    )
                )
            except (ChannelWrongStateError, AMQPConnectionError):
                self._reconnect()
                continue
            else:
                break


def get_amqp_client(host, queue_name, user=settings.RABBITMQ_USER,
                    password=settings.RABBITMQ_PASSWORD):
    """
    Возвращает канал связи с сервером AMQP, с заданной очередью сообщений.
    """
    try:
        return AMQPClient(host, queue_name, user, password)
    except pika.exceptions.AMQPConnectionError:
        pass
